@@ -1,6 +1,6 @@
 use pylon_cxx::NodeMap;
 
-fn main() -> Result<(), pylon_cxx::PylonError> {
+fn main() -> anyhow::Result<()> {
     // Before using any pylon methods, the pylon runtime must be initialized.
     let _pylon = pylon_cxx::PylonAutoInit::new();
 
