@@ -1,25 +1,47 @@
+#![cfg_attr(feature = "backtrace", feature(backtrace))]
+
+#[cfg(feature = "backtrace")]
+use std::backtrace::Backtrace;
+
 #[derive(Debug)]
-pub struct PylonError(String);
+pub struct PylonError {
+    msg: String,
+    #[cfg(feature = "backtrace")]
+    backtrace: Backtrace,
+}
 
 impl From<cxx::Exception> for PylonError {
     fn from(orig: cxx::Exception) -> PylonError {
-        PylonError(orig.what().into())
+        PylonError {
+            msg: orig.what().into(),
+            #[cfg(feature = "backtrace")]
+            backtrace: Backtrace::capture(),
+        }
     }
 }
 
 impl From<std::str::Utf8Error> for PylonError {
     fn from(_: std::str::Utf8Error) -> PylonError {
-        PylonError("Cannot convert C++ string to UTF-8".to_string())
+        PylonError {
+            msg: "Cannot convert C++ string to UTF-8".to_string(),
+            #[cfg(feature = "backtrace")]
+            backtrace: Backtrace::capture(),
+        }
     }
 }
 
 impl std::fmt::Display for PylonError {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
-        write!(f, "PylonError({})", self.0)
+        write!(f, "PylonError({})", self.msg)
     }
 }
 
-impl std::error::Error for PylonError {}
+impl std::error::Error for PylonError {
+    #[cfg(feature = "backtrace")]
+    fn backtrace(&self) -> Option<&std::backtrace::Backtrace> {
+        Some(&self.backtrace)
+    }
+}
 
 type PylonResult<T> = Result<T, PylonError>;
 
@@ -46,6 +68,7 @@ mod ffi {
         type CIntegerParameter;
         type CFloatParameter;
         type CEnumParameter;
+        type CCommandParameter;
 
         fn PylonInitialize();
         fn PylonTerminate(ShutDownLogging: bool);
@@ -92,6 +115,11 @@ mod ffi {
             name: &str,
         ) -> Result<UniquePtr<CEnumParameter>>;
 
+        fn node_map_get_command_parameter(
+            camera: &UniquePtr<CInstantCamera>,
+            name: &str,
+        ) -> Result<UniquePtr<CCommandParameter>>;
+
         fn boolean_node_get_value(boolean_node: &UniquePtr<CBooleanParameter>) -> Result<bool>;
         fn boolean_node_set_value(
             boolean_node: &UniquePtr<CBooleanParameter>,
@@ -118,6 +146,8 @@ mod ffi {
         ) -> Result<UniquePtr<CxxVector<CxxString>>>;
         fn enum_node_set_value(enum_node: &UniquePtr<CEnumParameter>, value: &str) -> Result<()>;
 
+        fn command_node_execute(node: &UniquePtr<CCommandParameter>, verify: bool) -> Result<()>;
+
         fn new_grab_result_ptr() -> Result<UniquePtr<CGrabResultPtr>>;
         fn grab_result_grab_succeeded(grab_result: &UniquePtr<CGrabResultPtr>) -> Result<bool>;
         fn grab_result_error_description(grab_result: &UniquePtr<CGrabResultPtr>)
@@ -155,7 +185,13 @@ pub struct PylonAutoInit {}
 impl PylonAutoInit {
     pub fn new() -> Self {
         ffi::PylonInitialize();
-        // println!("pylon initialized");
+        Self {}
+    }
+}
+
+impl Default for PylonAutoInit {
+    fn default() -> Self {
+        ffi::PylonInitialize();
         Self {}
     }
 }
@@ -163,7 +199,6 @@ impl PylonAutoInit {
 impl Drop for PylonAutoInit {
     fn drop(&mut self) {
         ffi::PylonTerminate(true);
-        // println!("pylon terminated");
     }
 }
 
@@ -215,10 +250,7 @@ impl Default for GrabOptions {
 
 impl GrabOptions {
     pub fn count(self, count: u32) -> GrabOptions {
-        Self {
-            count: Some(count),
-            ..self
-        }
+        Self { count: Some(count) }
     }
 }
 
@@ -307,11 +339,22 @@ impl EnumNode {
     }
 }
 
+pub struct CommandNode {
+    inner: cxx::UniquePtr<ffi::CCommandParameter>,
+}
+
+impl CommandNode {
+    pub fn execute(&self, verify: bool) -> PylonResult<()> {
+        ffi::command_node_execute(&self.inner, verify).into_rust()
+    }
+}
+
 pub trait NodeMap {
     fn boolean_node(&self, name: &str) -> PylonResult<BooleanNode>;
     fn integer_node(&self, name: &str) -> PylonResult<IntegerNode>;
     fn float_node(&self, name: &str) -> PylonResult<FloatNode>;
     fn enum_node(&self, name: &str) -> PylonResult<EnumNode>;
+    fn command_node(&self, name: &str) -> PylonResult<CommandNode>;
 }
 
 unsafe impl Send for InstantCamera {}
@@ -333,6 +376,10 @@ impl NodeMap for InstantCamera {
         let inner = ffi::node_map_get_enum_parameter(&self.inner, name)?;
         Ok(EnumNode { inner })
     }
+    fn command_node(&self, name: &str) -> PylonResult<CommandNode> {
+        let inner = ffi::node_map_get_command_parameter(&self.inner, name)?;
+        Ok(CommandNode { inner })
+    }
 }
 
 impl InstantCamera {
@@ -465,7 +512,6 @@ impl GrabResult {
     pub fn image_size(&self) -> PylonResult<u32> {
         ffi::grab_result_image_size(&self.inner).into_rust()
     }
-
 }
 
 trait CxxResultExt {
